// mfctestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "mfctest.h"
#include "mfctestDlg.h"
#include "dlgs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const char* strVDB = ".vdb";

class CVDBChooserDialog : public CFileDialog
{
	DECLARE_DYNAMIC(CVDBChooserDialog)

public:
	CVDBChooserDialog(
		LPCTSTR lpszFileName = NULL,
		LPCTSTR lpszTitle = NULL,
		DWORD dwFlags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST);
	virtual CString GetPathName() const;
	virtual CString GetFileName() const;
	virtual CString GetFileExt() const;
	virtual CString GetFileTitle() const;
	virtual void OnFolderChange();
	virtual void OnFileNameChange();
	virtual BOOL OnFileNameOK(){return 0;};

protected:
	CString m_szVDB;

protected:
	//{{AFX_MSG(CVDBChooserDialog)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CVDBChooserDialog
IMPLEMENT_DYNAMIC(CVDBChooserDialog, CFileDialog)


BEGIN_MESSAGE_MAP(CVDBChooserDialog, CFileDialog)
	//{{AFX_MSG_MAP(CVDBChooserDialog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CVDBChooserDialog::CVDBChooserDialog(LPCTSTR lpszFileName, 
									 LPCTSTR lpszTitle,
									 DWORD dwFlags) :
		CFileDialog(TRUE, 
			NULL, 
			lpszFileName, 
			dwFlags, 
			_T("VDB database (*.vdb)|*.vdb||"), 
			NULL)
{
	m_ofn.lpstrTitle = lpszTitle;
}

WNDPROC oldWindowProc; /* original window function to save*/
/* hooked window funcction */
static LRESULT CALLBACK SubclassWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(uMsg == WM_COMMAND){
		int nCtrlID = LOWORD(wParam);
		if(LOWORD(wParam) == IDOK)
		{
			// if OK button is pushed and text box is empty, enter dummy filename for end dialog
			char filename[_MAX_PATH];
			int nRead = GetDlgItemText(hwnd, edt1, filename, sizeof(filename));
			if( nRead > strlen(strVDB) && _strnicmp(&filename[nRead - strlen(strVDB)], strVDB, strlen(strVDB)) == 0 )
			{
				if( CFileDialog* pDlg = (CFileDialog*) CWnd::FromHandle(hwnd) )
					pDlg->EndDialog(IDOK);
			}
		}
	}
	return CallWindowProc(oldWindowProc, hwnd, uMsg, wParam, lParam);
}

BOOL CVDBChooserDialog::OnInitDialog() 
{
	CFileDialog::OnInitDialog();
	oldWindowProc = (WNDPROC)::SetWindowLong(GetSafeHwnd(), GWL_WNDPROC, (DWORD)SubclassWindowProc);
	return TRUE;
}

void CVDBChooserDialog::OnFileNameChange()
{
	CWnd* pParent = GetParent()->GetDlgItem(lst2); 

	AfxMessageBox(pParent ? "Has parent" : "no parent");
	if( pParent )
	{
		// get the list control 
		CListCtrl *pList = ( CListCtrl * ) pParent->GetDlgItem ( 1 ); 

		// currently selected item 
		int nPos = pList->GetNextItem ( -1, LVNI_ALL | LVNI_SELECTED ); 

		if( nPos != -1 )
		{
			CString szSelect = pList->GetItemText(nPos, 0);
			if( szSelect.Right(4).CompareNoCase(strVDB) == 0 )
			{
				TCHAR path[_MAX_PATH]; 
				GetCurrentDirectory(_MAX_PATH, path); 
				m_szVDB = path;
				m_szVDB += '\\';
				m_szVDB += szSelect;
				GetParent()->GetDlgItem(edt1)->SetWindowText(szSelect);
			}
		}
	}
}

void CVDBChooserDialog::OnFolderChange()
{
	CComboBox* pCombo = (CComboBox*) GetParent()->GetDlgItem(cmb2);
	int nPos = pCombo->GetCurSel();
	if( nPos != CB_ERR )
	{
		CString szFolder;
		pCombo->GetLBText(nPos, szFolder);
		if( szFolder.Right(4).CompareNoCase(strVDB) == 0 )
		{
			CDialog* pDialog = (CDialog*) GetParent();
			pDialog->EndDialog(IDOK);
		}
	}
}

CString CVDBChooserDialog::GetPathName() const 
{
	CString szPathName;
	//Add extension if needed
	if( !m_szVDB.IsEmpty() )
	{
		szPathName = m_szVDB;
		if( !szPathName.IsEmpty() && szPathName.Right(4).CompareNoCase(strVDB) != 0 ) szPathName += strVDB;
	}
	else
		szPathName = CFileDialog::GetPathName();
	return szPathName;
}

CString CVDBChooserDialog::GetFileName() const{
	CString pathname = GetPathName();
	int slashpos = pathname.ReverseFind('\\');
	return pathname.Mid(slashpos + 1);
}

CString CVDBChooserDialog::GetFileExt() const{
	CString filename = GetFileName();
	int dotpos = filename.ReverseFind('.');
	return (dotpos != -1) ? filename.Mid(dotpos+1) : "";
}

CString CVDBChooserDialog::GetFileTitle() const{
	CString filename = GetFileName();
	int dotpos = filename.ReverseFind('.');
	return filename.Left(dotpos);
}

/////////////////////////////////////////////////////////////////////////////
// CMfctestDlg dialog

CMfctestDlg::CMfctestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMfctestDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMfctestDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CMfctestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMfctestDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CMfctestDlg, CDialog)
	//{{AFX_MSG_MAP(CMfctestDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BTN_VDB, OnBtnVdb)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMfctestDlg message handlers

BOOL CMfctestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMfctestDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CMfctestDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CMfctestDlg::OnBtnVdb() 
{
	CVDBChooserDialog vdbDlg;
	vdbDlg.DoModal();
	TRACE("Vdb selection is %s\n", vdbDlg.GetPathName());
}
